precision mediump float;
varying vec2 textureCoordinate;
uniform sampler2D y_tex;
uniform sampler2D u_tex;
uniform sampler2D v_tex;

uniform highp vec2 singleStepOffset;
uniform highp vec4 params;
uniform highp float brightness;
const highp vec3 W = vec3(0.299, 0.587, 0.114);
const highp mat3 saturateMatrix = mat3(
     1.1102, -0.0598, -0.061,
     -0.0774, 1.0826, -0.1186,
     -0.0228, -0.0228, 1.1772);
varying highp vec2 blurCoordinates[12];

vec3 getColor(vec2 coordinate) {
     float y = texture2D(y_tex, coordinate).r;
     float u = texture2D(u_tex, coordinate).r - 0.5;
     float v = texture2D(v_tex, coordinate).r - 0.5;
     vec3 rgb = vec3(y + 1.403 * v, y - 0.344 * u - 0.714 * v, y + 1.77 * u);
     return rgb;
}

highp float hardLight(highp float color) {
     if (color <= 0.5)
         color = color * color * 2.0;
     else
         color = 1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
     return color;
}

void main() {

     highp vec3 centralColor = getColor(textureCoordinate).rgb;

     highp float sampleColor = centralColor.g * 22.0;
     sampleColor += getColor(blurCoordinates[0]).g * 2.0;
     sampleColor += getColor(blurCoordinates[1]).g * 2.0;
     sampleColor += getColor(blurCoordinates[2]).g * 2.0;
     sampleColor += getColor(blurCoordinates[3]).g * 2.0;
     sampleColor += getColor(blurCoordinates[4]).g * 2.0;
     sampleColor += getColor(blurCoordinates[5]).g * 2.0;
     sampleColor += getColor(blurCoordinates[6]).g * 2.0;
     sampleColor += getColor(blurCoordinates[7]).g * 2.0;
     sampleColor += getColor(blurCoordinates[8]).g * 3.0;
     sampleColor += getColor(blurCoordinates[9]).g * 3.0;
     sampleColor += getColor(blurCoordinates[10]).g * 3.0;
     sampleColor += getColor(blurCoordinates[11]).g * 3.0;

     sampleColor = sampleColor / 50.0;
     highp float highPass = centralColor.g - sampleColor + 0.5;

     highPass = hardLight(highPass);
     highPass = hardLight(highPass);
     highPass = hardLight(highPass);
     highPass = hardLight(highPass);
     highPass = hardLight(highPass);

     highp float lumance = dot(centralColor, W);

     highp float alpha = pow(lumance, params.r);

     highp vec3 smoothColor = centralColor + (centralColor-vec3(highPass))*alpha*0.1;

     smoothColor.r = clamp(pow(smoothColor.r, params.g), 0.0, 1.0);
     smoothColor.g = clamp(pow(smoothColor.g, params.g), 0.0, 1.0);
     smoothColor.b = clamp(pow(smoothColor.b, params.g), 0.0, 1.0);

     highp vec3 lvse = vec3(1.0)-(vec3(1.0)-smoothColor)*(vec3(1.0)-centralColor);
     highp vec3 bianliang = max(smoothColor, centralColor);
     highp vec3 rouguang = 2.0*centralColor*smoothColor + centralColor*centralColor- 2.0*centralColor*centralColor*smoothColor;

     gl_FragColor = vec4(mix(centralColor, lvse, alpha), 1.0);
     gl_FragColor.rgb = mix(gl_FragColor.rgb, bianliang, alpha);
     gl_FragColor.rgb = mix(gl_FragColor.rgb, rouguang, params.b);

     highp vec3 satcolor = gl_FragColor.rgb * saturateMatrix;
     gl_FragColor.rgb = mix(gl_FragColor.rgb, satcolor, params.a);
     gl_FragColor.rgb = vec3(gl_FragColor.rgb + vec3(brightness));
}