#extension GL_OES_EGL_image_external : require
precision mediump float;
varying vec2 textureCoordinate;
varying vec2 leftTextureCoordinate;
varying vec2 rightTextureCoordinate;

varying vec2 topTextureCoordinate;
varying vec2 topLeftTextureCoordinate;
varying vec2 topRightTextureCoordinate;

varying vec2 bottomTextureCoordinate;
varying vec2 bottomLeftTextureCoordinate;
varying vec2 bottomRightTextureCoordinate;
uniform samplerExternalOES inputImageTexture;

const highp vec3 W = vec3(0.2125, 0.7154, 0.0721);

void main() {
    float centerIntensity = dot(texture2D(inputImageTexture, textureCoordinate).rgb, W);
    float bottomLeftIntensity = dot(texture2D(inputImageTexture, bottomLeftTextureCoordinate).rgb, W);
    float topRightIntensity = dot(texture2D(inputImageTexture, topRightTextureCoordinate).rgb, W);
    float topLeftIntensity = dot(texture2D(inputImageTexture, topLeftTextureCoordinate).rgb, W);
    float bottomRightIntensity = dot(texture2D(inputImageTexture, bottomRightTextureCoordinate).rgb, W);
    float leftIntensity = dot(texture2D(inputImageTexture, leftTextureCoordinate).rgb, W);
    float rightIntensity = dot(texture2D(inputImageTexture, rightTextureCoordinate).rgb, W);
    float bottomIntensity = dot(texture2D(inputImageTexture, bottomTextureCoordinate).rgb, W);
    float topIntensity = dot(texture2D(inputImageTexture, topTextureCoordinate).rgb, W);
    float h = (-topLeftIntensity - 2.0 * topIntensity - topRightIntensity + bottomLeftIntensity + 2.0 * bottomIntensity + bottomRightIntensity);
    float v = (-bottomLeftIntensity - 2.0 * leftIntensity - topLeftIntensity + bottomRightIntensity + 2.0 * rightIntensity + topRightIntensity);

    float mag = 1.0 - length(vec2(h, v));

    float pixelIntensitySum = bottomIntensity + bottomLeftIntensity + bottomRightIntensity
                            + topIntensity + topLeftIntensity + topRightIntensity
                            + leftIntensity + rightIntensity + centerIntensity;
    float sumTest = step(1.8, pixelIntensitySum);
    float pixelTest = step(0.01, centerIntensity);
    mag = mix(mag, sumTest * pixelTest, 0.5);

    vec4 outColor = texture2D(inputImageTexture, textureCoordinate);
    outColor = vec4(
        mix(mag, outColor.r, 0.2),
        mix(mag, outColor.g, 0.2),
        mix(mag, outColor.b, 0.2),
        1.0
    );
    gl_FragColor = outColor;
}