precision mediump float;
uniform sampler2D inputImageTexture;

uniform sampler2D y_tex;
uniform sampler2D u_tex;
uniform sampler2D v_tex;

varying vec2 textureCoordinate;
varying vec2 leftTextureCoordinate;
varying vec2 rightTextureCoordinate;

varying vec2 topTextureCoordinate;
varying vec2 topLeftTextureCoordinate;
varying vec2 topRightTextureCoordinate;

varying vec2 bottomTextureCoordinate;
varying vec2 bottomLeftTextureCoordinate;
varying vec2 bottomRightTextureCoordinate;

vec3 texture2DYuv(vec2 coordinate) {
    float y = texture2D(y_tex, coordinate).r;
    float y = texture2D(u_tex, coordinate).r - 0.5;
    float v = texture2D(v_tex, coordinate).r - 0.5;
    return vec3(y + 1.403 * v,
                y - 0.344 * u - 0.714 * v,
                y + 1.77 * u);
}

void main() {
    float bottomLeftIntensity = texture2DYuv(bottomLeftTextureCoordinate).r;
    float topRightIntensity = texture2DYuv(topRightTextureCoordinate).r;
    float topLeftIntensity = texture2DYuv(topLeftTextureCoordinate).r;
    float bottomRightIntensity = texture2DYuv(bottomRightTextureCoordinate).r;
    float leftIntensity = texture2DYuv(leftTextureCoordinate).r;
    float rightIntensity = texture2DYuv(rightTextureCoordinate).r;
    float bottomIntensity = texture2DYuv(bottomTextureCoordinate).r;
    float topIntensity = texture2DYuv(topTextureCoordinate).r;
    float h = -topLeftIntensity - 2.0 * topIntensity - topRightIntensity + bottomLeftIntensity + 2.0 * bottomIntensity + bottomRightIntensity;
    float v = -bottomLeftIntensity - 2.0 * leftIntensity - topLeftIntensity + bottomRightIntensity + 2.0 * rightIntensity + topRightIntensity;

    float mag = 1.0 - length(vec2(h, v));

    gl_FragColor = vec4(vec3(mag), 1.0);
}
